﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace LEDEdit
{
    class LEDMatrix
    {
        public int MaxNum;
        public Boolean Type; //False = box True = circle
        public int SizeX;
        public int SizeY;
        List<Matrix1> Data;
        public int Color;

        public LEDMatrix()
        {
            // 8x8 monoColor 1Pattern
            SizeX = 8;
            SizeY = 8;
            Type = false;
            MaxNum = 1;
            Color = 0x00ff0000;
            Data = new List<Matrix1>();
            Data.Add(new Matrix1());
        }
        public LEDMatrix(int sx,int sy,Boolean type,int maxmun,int col)
        {
            // 8x8 monoColor 1Pattern
            SizeX = sx;
            SizeY = sy;
            Type = false;
            MaxNum = 1;
            Color = col;
            Data = new List<Matrix1>();
            Data.Add(new Matrix1(sx, sy, MaxNum));
        }
        public int GetMaxPattern() {return Data.Count;}

        Boolean IndexCheck(int idx)
        {
            if (idx >= 0 || idx <= Data.Count) return true;
            return false;
        }
        /// <summary>
        /// データを追加
        /// </summary>
        /// <returns>LastMatrix</returns>
        public Matrix1 Add(int sx,int sy,int num)
        {
            SizeX = sx;
            SizeY = sy;
            MaxNum = num;
            Data.Add(new Matrix1(sx, sy,num));
            return Data[Data.Count-1];
        }
        public Boolean Clear()
        {
            Data.Clear();
            return true;
        }
        public Boolean Remove(int idx)
        {
            if (!IndexCheck(idx)) return false;
            Data.RemoveAt(idx);
            return true;
        }
        public Boolean Insert(int idx,Matrix1 dat)
        {
            if (!IndexCheck(idx)) return false;
            Data.Insert(idx, dat);
            return true;
        }
        public Boolean Replace(int idx,Matrix1 dat)
        {
            if (!IndexCheck(idx)) return false;
            Data[idx].Copy(dat);
            return true;
        }
        public Matrix1 GetMatrix(int idx)
        {
            if (!IndexCheck(idx)) return null;
            return Data[idx];
        }
        public void GetPattern(int idx,Matrix1 dest)
        {
            if (!IndexCheck(idx)) return;
            dest.Copy(Data[idx]);
            return;
        }
        public void SaveCSV(String FilePath)
        {
            StreamWriter sw = new StreamWriter(FilePath);
            //Header
            sw.Write( Data.Count.ToString() + ","
                    + SizeX.ToString() + ","
                    + SizeY.ToString()+","
                    + MaxNum.ToString()+","
                    + Color.ToString()+"\r\n");
            
            for (int count = 0; count < Data.Count; count++)
            {
                sw.WriteLine(count.ToString());
                Data[count].ToStreamCSV(sw);
                
            }
            sw.Close();
        }        
        public void LoadCSV(String FilePath)
        {
            StreamReader sr = new StreamReader(FilePath);
            //Header データ数,サイズ　X,Y
            String wrkStr = sr.ReadLine();
            String [] param=wrkStr.Split(',');
            int[,] wrkDat;

            int DataCount = int.Parse(param[0]);
            SizeX = int.Parse(param[1]);
            SizeY = int.Parse(param[2]);
            MaxNum = int.Parse(param[3]);
            Color = int.Parse(param[4]);

            Data.Clear();

            for (int count = 0; count < DataCount; count++)
            {
                wrkDat = new int[SizeX, SizeY];
                wrkStr = sr.ReadLine(); //CountSkip 
                for(int cy=0;cy<SizeY;cy++)
                {                      
                    wrkStr = sr.ReadLine();
                    param=wrkStr.Split(',');

                    for(int cx=0;cx<SizeX;cx++)
                    {
                        wrkDat[cx,cy]=int.Parse(param[cx]);
                    }
                }
                Data.Add(new Matrix1(SizeX,SizeY,MaxNum,wrkDat));
            }
            sr.Close();
        }
        public String ToHexH(int idx)
        {
            if (!IndexCheck(idx)) return "err";
            return Data[idx].ToHexHorison();
        }
        public String ToHexV(int idx)
        {
            if (!IndexCheck(idx)) return "err";
            return Data[idx].ToHexVertical()+Data[idx].ToHexVertical2();
        }        
     }

    class Matrix1
    {
        int MaxNum;  //最大(色?)数
        int SizeX;
        int SizeY;
        int[,] Data;   

        public Matrix1()
        {
            // 8x8 monoColor 1Pattern
            SizeX = 8;
            SizeY = 8;
            MaxNum = 1;
            Data = new int[SizeX,SizeY];         
         }

        public Matrix1(int sx, int sy, int num, int[,] dat)
        {
            SizeX = sx;
            SizeY = sy;
            MaxNum = num;
            Data = dat;
        }
        public int GetSizeX() { return SizeX;}
        public int GetSizeY() { return SizeY;}

        /// <summary>
        /// 独自再定義用
        /// </summary>
        /// <param name="x">Width</param>
        /// <param name="y">Height</param>
        /// <param name="max">maxParam</param>
        public Matrix1(int x, int y,int max)
        {
            SizeX = x;
            SizeY = y;
            MaxNum = max;
            Data = new int[x, y];
        }
        public int[,]  GetMatrix(int idx)
        {
            return Data; 
        }
        public int GetXY(int x, int y)
        {
            if (x < 0 || x > SizeX) return -1;
            if (y < 0 || y > SizeY) return -1;

            return Data[x,y];
        }
        public Boolean SetXY(int x, int y, int num)
        {
            if (x < 0 || x > SizeX) return false;
            if (y < 0 || y > SizeY) return false;
            Data[x, y] = num;
            return true;
        }
        /// <summary>
        /// 内容を+1し最大数なら0に戻します
        /// </summary>
        /// <param name="x">pos_x</param>
        /// <param name="y">pos_y</param>
        /// <returns></returns>
        public Boolean PlusXY(int x, int y)
        {
            if (x < 0 || x >= SizeX) return false;
            if (y < 0 || y >= SizeY) return false;
            if(Data[x, y] >= MaxNum ) Data[x, y] = 0;
            else Data[x, y]++;
            return true;
        }
        public Boolean Fill(int num)
        {
            for (int y = 0; y < SizeY; y++)
            {
                for (int x = 0; x < SizeX; x++)
                {
                    Data[x,y] = num;
                }
            }
            return true;
        }
        public Boolean Clear()
        {
            return Fill(0);
        }
        public Boolean Copy(Matrix1 src)
        {
            for (int y = 0; y < SizeY; y++)
            {
                for (int x = 0; x < SizeX; x++)
                {
                    Data[x, y] = src.GetXY(x,y);
                }
            }
            return true;
        }
        public void RotateUp()
        {
            int[] work = new int[SizeX];
            for (int x = 0; x < SizeY; x++)
            {
                work[x] = Data[x, 0];
            }
            for (int y = 1; y < SizeY; y++)
            {
                for (int x = 0; x < SizeX; x++)
                {
                    Data[x, y - 1] = Data[x, y];
                }
            }
            for (int x = 0; x < SizeY; x++)
            {
                Data[x, SizeY - 1] = work[x];
            }
        }
        public void RotateDown()
        {
            int[] work = new int[SizeX];
            for (int x = 0; x < SizeY; x++)
            {
                work[x] = Data[x, SizeY - 1];
            }
            for (int y = SizeY - 2; y >= 0; y--)
            {
                for (int x = 0; x < SizeX; x++)
                {
                    Data[x, y + 1] = Data[x, y];
                }
            }
            for (int x = 0; x < SizeY; x++)
            {
                Data[x, 0] = work[x];
            }
        }
        public void RotateLeft()
        {
            int[] work = new int[SizeY];
            for (int y = 0; y < SizeY; y++)
            {
                work[y] = Data[0, y];
            }
            for (int y = 0; y < SizeY; y++)
            {
                for (int x = 1; x < SizeX; x++)
                {
                    Data[x - 1, y] = Data[x, y];
                }
            }
            for (int y = 0; y < SizeY; y++)
            {
                Data[SizeX - 1, y] = work[y];
            }
        }
        public void RotateRight()
        {
            int[] work = new int[SizeY];
            for (int y = 0; y < SizeY; y++)
            {
                work[y] = Data[SizeX - 1, y];
            }
            for (int y = 0; y < SizeY; y++)
            {
                for (int x = SizeX - 2; x >= 0; x--)
                {
                    Data[x + 1, y] = Data[x, y];
                }
            }
            for (int y = 0; y < SizeY; y++)
            {
                Data[0, y] = work[y];
            }
        }
        public void ReverseH()
        {
            int[,] work=new int[SizeX,SizeY];
            for (int y = 0; y < SizeY ; y++)
            {
                for (int x = 0; x < SizeX ; x++)
                {
                    work[SizeX-x-1, y] = Data[x, y];
                }
            }
            Data = work;
            
        }
        public void ReverseV()
        {
            int[,] work = new int[SizeX, SizeY];
            for (int y = 0; y < SizeY; y++)
            {
                for (int x = 0; x < SizeX; x++)
                {
                    work[x, SizeY-y-1] = Data[x, y];
                }
            }
            Data = work;
        }
        public void AllPlus()
        {
            for (int y = 0; y < SizeY; y++)
            {
                for (int x = 0; x < SizeX; x++)
                {
                    Data[x, y]++;
                    if (Data[x, y] > MaxNum) Data[x, y] = 0;
                }
            }
        
        }
        public String ToString(String SplitString)
        {
            String workStr="";
            for (int y = 0; y <= SizeY; y++)
            {
                for (int x = 0; x <= SizeX; x++)
                {
                    workStr += Data[x,y].ToString();
                }
                workStr += "\n";
            }
            return workStr;
            
        }
        public String ToStreamCSV(StreamWriter sw)
        {
            String workStr = "";
            for (int y = 0; y < SizeY; y++)
            {
                for (int x = 0; x < SizeX; x++)
                {
                    workStr += Data[x, y].ToString();
                    if (x != SizeX-1) workStr += ","; //Spliter
                }
                workStr += "\r\n";
                sw.Write(workStr);
                workStr = "";
            }
            return workStr;

        }
        public String ToHexHorison()
        {
            String retStr="";
            int[] iwork=new int[SizeX+7];//8の倍数に揃えるためのワーク

            int res = 255;
            
            //iWork0 初期化
            //iWorkにDataから必要部分抜きだし(MSB 左詰)
            //Horizon 左上Bit8→1繰返
            for (int dy = 0; dy < SizeY; dy++)
            {
                for (int bit = 0; bit < iwork.Length; bit++)
                {
                    if (bit < SizeX) iwork[bit] = Data[bit, dy];
                    else iwork[bit] = 0;//余分を0初期化
                }

                for (int bit8 = 0; bit8 < (iwork.Length / 8); bit8++)
                {
                    res = iwork[bit8 * 8] * 128
                        + iwork[bit8 * 8 + 1] * 64
                        + iwork[bit8 * 8 + 2] * 32
                        + iwork[bit8 * 8 + 3] * 16
                        + iwork[bit8 * 8 + 4] * 8
                        + iwork[bit8 * 8 + 5] * 4
                        + iwork[bit8 * 8 + 6] * 2
                        + iwork[bit8 * 8 + 7];
                    retStr += "0x" + res.ToString("x2")+","; //16進1桁
                }
            }
            return retStr;
        }
        public String ToHexHorison2()
        {
            String retStr = "";
            int[] iwork = new int[SizeX + 7];//8の倍数に揃えるためのワーク

            int res = 255;

            //iWork0 初期化
            //iWorkにDataから必要部分抜きだし(MSB 左詰)
            //Horizon 左上Bit8→1繰返
            for (int dy = 0; dy < SizeY; dy++)
            {
                for (int bit = 0; bit < iwork.Length; bit++)
                {
                    if (bit < SizeX) iwork[bit] = Data[bit, dy];
                    else iwork[bit] = 0;//余分を0初期化
                }

                for (int bit8 = 0; bit8 < (iwork.Length / 8); bit8++)
                {
                    res = iwork[bit8 * 8]
                        + iwork[bit8 * 8 + 1] * 2
                        + iwork[bit8 * 8 + 2] * 4
                        + iwork[bit8 * 8 + 3] * 8
                        + iwork[bit8 * 8 + 4] * 16
                        + iwork[bit8 * 8 + 5] * 32
                        + iwork[bit8 * 8 + 6] * 64
                        + iwork[bit8 * 8 + 7] * 128;
                    retStr += "0x" + res.ToString("x2") + ","; //16進1桁
                }
            }
            return retStr;
        }
        public String ToHexVertical()
        {
            String retStr = "";
            int[] iwork = new int[SizeX + 7];//8の倍数に揃えるためのワーク

            int res = 255;

            for (int dx = 0; dx < SizeX; dx++)
            {
                //iWorkにDataから必要部分抜きだし(MSB 左詰)
                //Vircical 左下Bit8→繰返
                for (int bit = 0; bit < iwork.Length; bit++)
                {
                    if (bit < SizeX) iwork[SizeY - 1 - bit] = Data[dx, bit];
                    else iwork[bit] = 0;//余分を0初期化
                }
                for (int bit8 = 0; bit8 < (iwork.Length / 8); bit8++)
                {
                    res = iwork[bit8 * 8] * 128
                        + iwork[bit8 * 8 + 1] * 64
                        + iwork[bit8 * 8 + 2] * 32
                        + iwork[bit8 * 8 + 3] * 16
                        + iwork[bit8 * 8 + 4] * 8
                        + iwork[bit8 * 8 + 5] * 4
                        + iwork[bit8 * 8 + 6] * 2
                        + iwork[bit8 * 8 + 7];
                retStr += "0x" + res.ToString("x2")+","; //16進1桁
                }
            }
            return retStr;
        }
        public String ToHexVertical2()
        {
            String retStr = "";
            int[] iwork = new int[SizeX + 7];//8の倍数に揃えるためのワーク

            int res = 255;

            for (int dx = 0; dx < SizeX; dx++)
            {
                //iWorkにDataから必要部分抜きだし(MSB 左詰)
                //Vircical 左下Bit8→繰返
                for (int bit = 0; bit < iwork.Length; bit++)
                {
                    if (bit < SizeX) iwork[SizeY - 1 - bit] = Data[dx, bit];
                    else iwork[bit] = 0;//余分を0初期化
                }
                for (int bit8 = 0; bit8 < (iwork.Length / 8); bit8++)
                {
                    res = iwork[bit8 * 8]
                        + iwork[bit8 * 8 + 1] * 2
                        + iwork[bit8 * 8 + 2] * 4
                        + iwork[bit8 * 8 + 3] * 8
                        + iwork[bit8 * 8 + 4] * 16
                        + iwork[bit8 * 8 + 5] * 32
                        + iwork[bit8 * 8 + 6] * 64
                        + iwork[bit8 * 8 + 7] * 128;
                    retStr += "0x" + res.ToString("x2") + ","; //16進1桁
                }
            }
            return retStr;
        }


    }
}
